@@ -20,6 +20,7 @@
 #define NUM_THREADS_FACTOR 0.7		/* defalt number of threads = real number of thread * THREADS_FACTOR */
 #define LLUI long long unsigned int
 #define MAX_FILTER_LEN 1024
+#define MAX_ERRBUF_LEN 1024
 
 /* global variable */
 //lnf_mem_t *memp;
@@ -61,10 +62,17 @@ int process_file(char *filename, lnf_filter_t *filterp) {
 
 	tid = (int)pthread_self();
 
+	if (strcmp(filename, "-") == 0) {
+		if (lnf_open(&filep, NULL, LNF_READ, NULL) != LNF_OK) {
+			fprintf(stderr, "[#%x] Can not open stdin\n", tid);
+			return 0;
+		}
+	} else {
+		if (lnf_open(&filep, filename, LNF_READ, NULL) != LNF_OK) {
+			fprintf(stderr, "[#%x] Can not open file %s\n", tid, filename);
+			return 0;
+		}
 
-	if (lnf_open(&filep, filename, LNF_READ, NULL) != LNF_OK) {
-		fprintf(stderr, "[#%x] Can not open file %s\n", tid, filename);
-		return 0;
 	}
 
 	lnf_rec_init(&recp);
@@ -151,6 +159,7 @@ int main(int argc, char **argv) {
 	int c, o;
 	int numaflags = 0;
 	output_t *outputp;
+	char errbuf[MAX_ERRBUF_LEN];
 //	lnf_filter_t *filterp;
 	
 
@@ -268,7 +277,7 @@ int main(int argc, char **argv) {
 			} 
 			strcat(filter, argv[optind]);
 			strcat(filter, " ");
-			printf("filter: %s \n", filter);
+			//printf("filter: %s \n", filter);
 		}
 
 		filterp = NULL;
@@ -289,7 +298,8 @@ int main(int argc, char **argv) {
 		}
 					
 		if (filterp == NULL) {
-			fprintf(stderr, "Can not compile filter: %s\n", filter);
+			lnf_error(errbuf, sizeof(errbuf) - 1);
+			fprintf(stderr, "Can not compile filter: %s\n%s\n", filter, errbuf);
 			exit(1);
 		}
 //		lnf_filter_free(filterp);
