@@ -64,6 +64,7 @@ typedef struct lnf_brec1_s {
 
 #define LNF_MAX_STRING		512
 #define LNF_MAX_RAW_LEN 	1024
+#define LNF_MAX_FIELD_LEN 	512		/* maximum length of data */
 
 /* type of fields */
 /* note: if the fields type allows two kind of data type  */
@@ -265,6 +266,18 @@ typedef void lnf_mem_cursor_t;
 #define LNF_INFO_BYTES			0x1A	/* summary of stored bytes - uint64_t */
 #define LNF_INFO_PACKETS		0x1C	/* summary of stored packets - uint64_t */
 #define LNF_INFO_PROC_BLOCKS	0x1E	/* number of processed blocks - uint64_t */
+#define LNF_INFO_FLOWS_TCP		0x20	/* number of stored TCP flows - uint64_t */
+#define LNF_INFO_FLOWS_UDP		0x22	/* number of stored UDP flows - uint64_t */
+#define LNF_INFO_FLOWS_ICMP		0x24	/* number of stored ICMP flows - uint64_t */
+#define LNF_INFO_FLOWS_OTHER	0x26	/* number of stored other flows - uint64_t */
+#define LNF_INFO_BYTES_TCP		0x28	/* number of stored TCP bytes - uint64_t */
+#define LNF_INFO_BYTES_UDP		0x2A	/* number of stored UDP bytes - uint64_t */
+#define LNF_INFO_BYTES_ICMP		0x2C	/* number of stored ICMP bytes - uint64_t */
+#define LNF_INFO_BYTES_OTHER	0x2E	/* number of stored other bytes - uint64_t */
+#define LNF_INFO_PACKETS_TCP	0x30	/* number of stored TCP packets - uint64_t */
+#define LNF_INFO_PACKETS_UDP	0x32	/* number of stored UDP packets - uint64_t */
+#define LNF_INFO_PACKETS_ICMP	0x34	/* number of stored ICMP packets - uint64_t */
+#define LNF_INFO_PACKETS_OTHER	0x36	/* number of stored other packets - uint64_t */
 
 
 /*! 
@@ -368,6 +381,18 @@ Get detailed information and statistics related to the open file.
 	LNF_INFO_BYTES - summary of stored bytes (uint64_t) \n
 	LNF_INFO_PACKETS - summary of stored packets (uint64_t) \n
 	LNF_INFO_PROC_BLOCKS - number of processed blocks (uint64_t) \n
+	LNF_INFO_FLOWS_TCP - number of stored TCP flows (uint64_t) \n
+	LNF_INFO_FLOWS_UDP - number of stored UDP flows (uint64_t) \n
+	LNF_INFO_FLOWS_ICMP - number of stored ICMP flows (uint64_t) \n
+	LNF_INFO_FLOWS_OTHER - number of stored other flows (uint64_t) \n
+	LNF_INFO_BYTES_TCP - number of stored TCP bytes (uint64_t) \n
+	LNF_INFO_BYTES_UDP - number of stored UDP bytes (uint64_t) \n
+	LNF_INFO_BYTES_ICMP - number of stored ICMP bytes (uint64_t) \n
+	LNF_INFO_BYTES_OTHER - number of stored other bytes (uint64_t) \n
+	LNF_INFO_PACKETS_TCP - number of stored TCP packets (uint64_t) \n
+	LNF_INFO_PACKETS_UDP - number of stored UDP packets (uint64_t) \n
+	LNF_INFO_PACKETS_ICMP - number of stored ICMP packets (uint64_t) \n
+	LNF_INFO_PACKETS_OTHER - number of stored other packets (uint64_t) \n
 \param *data 		pointer initialized and zeroed data structure
 \param size			maximum size allocated for *data structure
 \return 			LNF_OK, LNF_ERR_NOMEM, LNF_ERR_OTHER
@@ -430,6 +455,49 @@ int lnf_rec_fset(lnf_rec_t *rec, int field, void *data);
 */
 int lnf_rec_fget(lnf_rec_t *rec, int field, void *data);
 
+#define LNF_REC_RAW_TLV 1				/* TLV version identification */
+#define LNF_REC_RAW_TLV_BUFSIZE 1024	/* recomended size of the buffer */
+/*!	\ingroup record
+\brief Fill buffer with binnary representation of record.
+
+Returns binnary representation of the record. The format of data 
+depend on the type of the record. The data cen be converted 
+back to the lnf_rec_t by the lnf_rec_set_raw function. 
+
+Currently on LNF_REC_RAW_TLV records are supported. This formats 
+encodes set fields in TLV (Type-Lenght-Value) format. The format 
+is platform independens so it can be used to transfer data between 
+differend platforms. 
+
+The data are filled to the pre-prepared buffer. If there is not enough 
+space in the buffer the LNF_ERR_NOMEM error code is returned. The recomended 
+size of the buffer is defined in LNF_REC_RAW_TLV_BUFSIZE 
+
+The example of use of lnf_rec_get_raw and lnf_rec_set_raw can be 
+found in examples/lnf_ex12_rec_raw.c.
+
+\param *rec 	pointer to lnf_rec_t structure 
+\param type 	currently only LNF_REC_RAW_TLV is suported
+\param *buf 	pointer to allocated buffer 
+\param size 	size of the buffer 
+\param ret_size the size of returned data in buffer 
+\return 			LNF_OK, LNF_ERR_OTHER, LNF_ERR_NOMEM
+*/
+int lnf_rec_get_raw(lnf_rec_t *rec, int type, char *buf, size_t size, size_t *ret_size);
+
+/*!	\ingroup record
+\brief Fill lnf_rec_t withe data from buffer gotten by lnf_rec_get_raw.
+
+The opposite function to lnf_rec_get_raw. For more information see lnf_rec_get_raw.
+
+\param *rec 	pointer to lnf_rec_t structure 
+\param *buf 	pointer to buffer with raw data
+\param size 	size of the data/buffer
+\return 			LNF_OK, LNF_ERR_OTHER, LNF_ERR_NOMEM
+*/
+int lnf_rec_set_raw(lnf_rec_t *rec, char *buf, size_t size);
+
+
 /*!	\ingroup record
 \brief Close file and release resources.
 
@@ -439,6 +507,8 @@ Close previously record object and release all relevant resources.
 */
 void lnf_rec_free(lnf_rec_t *rec);
 
+
+
 /*!	\ingroup filter
 \brief Initialize empty filter object.
 
@@ -508,6 +578,21 @@ See comment in lnf_filter_init function for more information.
 */
 int	lnf_filter_init_v2(lnf_filter_t **filterp, char *expr);
 
+/*!	\ingroup filter
+\brief Returns pointer to libnf-ffilter (ff_t) internal structure.
+
+In cases when the internal strcture of ffilter must be accessed
+from the outside of the library (index optimalisation etc.).
+
+The pointer is returned onlu if filter was properly initialised as
+libnv v2 filter {via. lnf_filter_init_v2). For other cases the NULL
+pointer is returned.
+
+\param *filter		double pointer to lnf_filter_t structure 
+\return 			void ponter to ffilter internal structute (ff_t).
+*/
+void *lnf_filter_ffilter_ptr(lnf_filter_t *filter);
+
 /*!	\ingroup filter
 \brief Match record object against filter.
 
@@ -803,6 +888,15 @@ Use lnf_mem_first_c instead.
 */
 void lnf_mem_read_reset(lnf_mem_t *lnf_mem);
 
+/*!	\ingroup memheap
+\brief Clean all data in memheap. 
+
+The memheap will be in the same state as it was after the initialisation. 
+
+\param *lnf_mem 	pointer to lnf_mem_t structure 
+*/
+void lnf_mem_clean(lnf_mem_t *lnf_mem);
+
 /*!	\ingroup memheap
 \brief Close memheap and release resources.
 
