@@ -75,20 +75,18 @@
 	#include <stdio.h>
 	#include "ffilter.h"
 	#include "ffilter_internal.h"
+	#include <string.h>
 
 	#define YY_EXTRA_TYPE ff_t
 
-//	int ff2_lex();
-
-	void yyerror(yyscan_t scanner, ff_t *filter, char *msg) {
-
-		ff_set_error(filter, msg);
+	void yyerror(yyscan_t scanner, ff_t *filter, char *msg)
+	{
+	    ff_set_error(filter, msg);
 	}
 
 
-
 /* Line 371 of yacc.c  */
-#line 92 "ffilter/ffilter_gram.c"
+#line 90 "ffilter/ffilter_gram.c"
 
 # ifndef YY_NULL
 #  if defined __cplusplus && 201103L <= __cplusplus
@@ -127,24 +125,46 @@ extern int ff2_debug;
      AND = 258,
      OR = 259,
      NOT = 260,
-     EQ = 261,
-     LT = 262,
-     GT = 263,
-     LP = 264,
-     RP = 265,
-     STRING = 266
+     ANY = 261,
+     EXIST = 262,
+     EQ = 263,
+     LT = 264,
+     GT = 265,
+     ISSET = 266,
+     LP = 267,
+     RP = 268,
+     LPS = 269,
+     RPS = 270,
+     IN = 271,
+     STRING = 272,
+     DIR = 273,
+     BIDIR_AND = 274,
+     BIDIR_OR = 275,
+     DIR_DIR_MAC = 276,
+     BAD_TOKEN = 277
    };
 #endif
 /* Tokens.  */
 #define AND 258
 #define OR 259
 #define NOT 260
-#define EQ 261
-#define LT 262
-#define GT 263
-#define LP 264
-#define RP 265
-#define STRING 266
+#define ANY 261
+#define EXIST 262
+#define EQ 263
+#define LT 264
+#define GT 265
+#define ISSET 266
+#define LP 267
+#define RP 268
+#define LPS 269
+#define RPS 270
+#define IN 271
+#define STRING 272
+#define DIR 273
+#define BIDIR_AND 274
+#define BIDIR_OR 275
+#define DIR_DIR_MAC 276
+#define BAD_TOKEN 277
 
 
 
@@ -152,7 +172,7 @@ extern int ff2_debug;
 typedef union YYSTYPE
 {
 /* Line 387 of yacc.c  */
-#line 47 "ffilter/ffilter_gram.y"
+#line 45 "ffilter/ffilter_gram.y"
 
 	uint64_t	t_uint;
 	double		t_double;
@@ -161,7 +181,7 @@ typedef union YYSTYPE
 
 
 /* Line 387 of yacc.c  */
-#line 165 "ffilter/ffilter_gram.c"
+#line 185 "ffilter/ffilter_gram.c"
 } YYSTYPE;
 # define YYSTYPE_IS_TRIVIAL 1
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
@@ -188,7 +208,7 @@ int ff2_parse ();
 /* Copy the second part of user declarations.  */
 
 /* Line 390 of yacc.c  */
-#line 192 "ffilter/ffilter_gram.c"
+#line 212 "ffilter/ffilter_gram.c"
 
 #ifdef short
 # undef short
@@ -415,22 +435,22 @@ union yyalloc
 #endif /* !YYCOPY_NEEDED */
 
 /* YYFINAL -- State number of the termination state.  */
-#define YYFINAL  12
+#define YYFINAL  20
 /* YYLAST -- Last index in YYTABLE.  */
-#define YYLAST   21
+#define YYLAST   41
 
 /* YYNTOKENS -- Number of terminals.  */
-#define YYNTOKENS  12
+#define YYNTOKENS  23
 /* YYNNTS -- Number of nonterminals.  */
-#define YYNNTS  3
+#define YYNNTS  7
 /* YYNRULES -- Number of rules.  */
-#define YYNRULES  11
+#define YYNRULES  26
 /* YYNRULES -- Number of states.  */
-#define YYNSTATES  21
+#define YYNSTATES  39
 
 /* YYTRANSLATE(YYLEX) -- Bison symbol number corresponding to YYLEX.  */
 #define YYUNDEFTOK  2
-#define YYMAXUTOK   266
+#define YYMAXUTOK   277
 
 #define YYTRANSLATE(YYX)						\
   ((unsigned int) (YYX) <= YYMAXUTOK ? yytranslate[YYX] : YYUNDEFTOK)
@@ -464,7 +484,8 @@ static const yytype_uint8 yytranslate[] =
        2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
        2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
        2,     2,     2,     2,     2,     2,     1,     2,     3,     4,
-       5,     6,     7,     8,     9,    10,    11
+       5,     6,     7,     8,     9,    10,    11,    12,    13,    14,
+      15,    16,    17,    18,    19,    20,    21,    22
 };
 
 #if YYDEBUG
@@ -472,24 +493,29 @@ static const yytype_uint8 yytranslate[] =
    YYRHS.  */
 static const yytype_uint8 yyprhs[] =
 {
-       0,     0,     3,     5,     6,     9,    13,    17,    21,    24,
-      28,    32
+       0,     0,     3,     5,     6,     8,    11,    14,    17,    20,
+      22,    25,    27,    30,    34,    38,    42,    44,    47,    51,
+      55,    58,    61,    63,    65,    67,    69
 };
 
 /* YYRHS -- A `-1'-separated list of the rules' RHS.  */
 static const yytype_int8 yyrhs[] =
 {
-      13,     0,    -1,    14,    -1,    -1,     5,    14,    -1,    14,
-       3,    14,    -1,    14,     4,    14,    -1,     9,    14,    10,
-      -1,    11,    11,    -1,    11,     6,    11,    -1,    11,     7,
-      11,    -1,    11,     8,    11,    -1
+      24,     0,    -1,    27,    -1,    -1,    17,    -1,    18,    17,
+      -1,    20,    17,    -1,    19,    17,    -1,    21,    17,    -1,
+      17,    -1,    17,    17,    -1,     6,    -1,     5,    27,    -1,
+      27,     3,    27,    -1,    27,     4,    27,    -1,    12,    27,
+      13,    -1,    25,    -1,     7,    25,    -1,    25,    29,    26,
+      -1,    25,    16,    28,    -1,    17,    28,    -1,    17,    15,
+      -1,    11,    -1,     8,    -1,     9,    -1,    10,    -1,    -1
 };
 
 /* YYRLINE[YYN] -- source line where rule number YYN was defined.  */
 static const yytype_uint8 yyrline[] =
 {
-       0,    67,    67,    68,    72,    73,    74,    75,    76,    77,
-      78,    79
+       0,    70,    70,    71,    75,    76,    77,    78,    79,    83,
+      84,    88,    89,    90,    91,    92,    93,   110,   111,   112,
+     116,   117,   121,   122,   123,   124,   125
 };
 #endif
 
@@ -498,8 +524,10 @@ static const yytype_uint8 yyrline[] =
    First, the terminals, then, starting at YYNTOKENS, nonterminals.  */
 static const char *const yytname[] =
 {
-  "$end", "error", "$undefined", "AND", "OR", "NOT", "EQ", "LT", "GT",
-  "LP", "RP", "STRING", "$accept", "filter", "expr", YY_NULL
+  "$end", "error", "$undefined", "AND", "OR", "NOT", "ANY", "EXIST", "EQ",
+  "LT", "GT", "ISSET", "LP", "RP", "LPS", "RPS", "IN", "STRING", "DIR",
+  "BIDIR_AND", "BIDIR_OR", "DIR_DIR_MAC", "BAD_TOKEN", "$accept", "filter",
+  "field", "value", "expr", "list", "cmp", YY_NULL
 };
 #endif
 
@@ -509,22 +537,25 @@ static const char *const yytname[] =
 static const yytype_uint16 yytoknum[] =
 {
        0,   256,   257,   258,   259,   260,   261,   262,   263,   264,
-     265,   266
+     265,   266,   267,   268,   269,   270,   271,   272,   273,   274,
+     275,   276,   277
 };
 # endif
 
 /* YYR1[YYN] -- Symbol number of symbol that rule YYN derives.  */
 static const yytype_uint8 yyr1[] =
 {
-       0,    12,    13,    13,    14,    14,    14,    14,    14,    14,
-      14,    14
+       0,    23,    24,    24,    25,    25,    25,    25,    25,    26,
+      26,    27,    27,    27,    27,    27,    27,    27,    27,    27,
+      28,    28,    29,    29,    29,    29,    29
 };
 
 /* YYR2[YYN] -- Number of symbols composing right hand side of rule YYN.  */
 static const yytype_uint8 yyr2[] =
 {
-       0,     2,     1,     0,     2,     3,     3,     3,     2,     3,
-       3,     3
+       0,     2,     1,     0,     1,     2,     2,     2,     2,     1,
+       2,     1,     2,     3,     3,     3,     1,     2,     3,     3,
+       2,     2,     1,     1,     1,     1,     0
 };
 
 /* YYDEFACT[STATE-NAME] -- Default reduction number in state STATE-NUM.
@@ -532,64 +563,71 @@ static const yytype_uint8 yyr2[] =
    means the default is an error.  */
 static const yytype_uint8 yydefact[] =
 {
-       3,     0,     0,     0,     0,     2,     4,     0,     0,     0,
-       0,     8,     1,     0,     0,     7,     9,    10,    11,     5,
-       6
+       3,     0,    11,     0,     0,     4,     0,     0,     0,     0,
+       0,    16,     2,    12,    17,     0,     5,     7,     6,     8,
+       1,    23,    24,    25,    22,     0,     0,     0,     0,    15,
+       0,    19,     9,    18,    13,    14,    21,    20,    10
 };
 
 /* YYDEFGOTO[NTERM-NUM].  */
 static const yytype_int8 yydefgoto[] =
 {
-      -1,     4,     5
+      -1,    10,    11,    33,    12,    31,    26
 };
 
 /* YYPACT[STATE-NUM] -- Index in YYTABLE of the portion describing
    STATE-NUM.  */
-#define YYPACT_NINF -5
+#define YYPACT_NINF -16
 static const yytype_int8 yypact[] =
 {
-      -3,    -3,    -3,     8,     5,    14,    -5,     0,    -4,    -2,
-       9,    -5,    -5,    -3,    -3,    -5,    -5,    -5,    -5,    -5,
-      18
+       0,     0,   -16,    15,     0,   -16,   -13,    -8,    11,    12,
+      16,    14,    10,   -16,   -16,    -2,   -16,   -16,   -16,   -16,
+     -16,   -16,   -16,   -16,   -16,    20,    21,     0,     0,   -16,
+      -7,   -16,    22,   -16,   -16,    37,   -16,   -16,   -16
 };
 
 /* YYPGOTO[NTERM-NUM].  */
 static const yytype_int8 yypgoto[] =
 {
-      -5,    -5,    -1
+     -16,   -16,    38,   -16,    -1,   -15,   -16
 };
 
 /* YYTABLE[YYPACT[STATE-NUM]].  What to do in state STATE-NUM.  If
    positive, shift that token.  If negative, reduce the rule which
    number is the opposite.  If YYTABLE_NINF, syntax error.  */
-#define YYTABLE_NINF -1
-static const yytype_uint8 yytable[] =
+#define YYTABLE_NINF -27
+static const yytype_int8 yytable[] =
 {
-       6,     7,     1,    13,    14,    12,     2,    16,     3,    17,
-      15,     0,    19,    20,     8,     9,    10,    13,    14,    11,
-      18,    13
+      13,    27,    28,    15,    16,     1,     2,     3,    36,    17,
+      30,    29,     4,    27,    28,    37,    20,     5,     6,     7,
+       8,     9,    21,    22,    23,    24,    34,    35,    18,    19,
+      25,   -26,     5,     6,     7,     8,     9,    30,    32,    38,
+      27,    14
 };
 
 #define yypact_value_is_default(Yystate) \
-  (!!((Yystate) == (-5)))
+  (!!((Yystate) == (-16)))
 
 #define yytable_value_is_error(Yytable_value) \
   YYID (0)
 
-static const yytype_int8 yycheck[] =
+static const yytype_uint8 yycheck[] =
 {
-       1,     2,     5,     3,     4,     0,     9,    11,    11,    11,
-      10,    -1,    13,    14,     6,     7,     8,     3,     4,    11,
-      11,     3
+       1,     3,     4,     4,    17,     5,     6,     7,    15,    17,
+      17,    13,    12,     3,     4,    30,     0,    17,    18,    19,
+      20,    21,     8,     9,    10,    11,    27,    28,    17,    17,
+      16,    17,    17,    18,    19,    20,    21,    17,    17,    17,
+       3,     3
 };
 
 /* YYSTOS[STATE-NUM] -- The (internal number of the) accessing
    symbol of state STATE-NUM.  */
 static const yytype_uint8 yystos[] =
 {
-       0,     5,     9,    11,    13,    14,    14,    14,     6,     7,
-       8,    11,     0,     3,     4,    10,    11,    11,    11,    14,
-      14
+       0,     5,     6,     7,    12,    17,    18,    19,    20,    21,
+      24,    25,    27,    27,    25,    27,    17,    17,    17,    17,
+       0,     8,     9,    10,    11,    16,    29,     3,     4,    13,
+      17,    28,    17,    26,    27,    27,    15,    28,    17
 };
 
 #define yyerrok		(yyerrstatus = 0)
@@ -1408,67 +1446,173 @@ yyreduce:
     {
         case 2:
 /* Line 1787 of yacc.c  */
-#line 67 "ffilter/ffilter_gram.y"
+#line 70 "ffilter/ffilter_gram.y"
     { filter->root = (yyvsp[(1) - (1)].node); }
     break;
 
   case 3:
 /* Line 1787 of yacc.c  */
-#line 68 "ffilter/ffilter_gram.y"
+#line 71 "ffilter/ffilter_gram.y"
     { filter->root = NULL; }
     break;
 
   case 4:
 /* Line 1787 of yacc.c  */
-#line 72 "ffilter/ffilter_gram.y"
-    { (yyval.node) = ff_new_node(scanner, filter, NULL, FF_OP_NOT, (yyvsp[(2) - (2)].node)); if ((yyval.node) == NULL) { YYABORT; }; }
+#line 75 "ffilter/ffilter_gram.y"
+    { strncpy((yyval.string), (yyvsp[(1) - (1)].string), FF_MAX_STRING - 1); }
     break;
 
   case 5:
 /* Line 1787 of yacc.c  */
-#line 73 "ffilter/ffilter_gram.y"
-    { (yyval.node) = ff_new_node(scanner, filter, (yyvsp[(1) - (3)].node), FF_OP_AND, (yyvsp[(3) - (3)].node)); if ((yyval.node) == NULL) { YYABORT; }; }
+#line 76 "ffilter/ffilter_gram.y"
+    { snprintf((yyval.string), FF_MAX_STRING - 1, "%s %s", (yyvsp[(1) - (2)].string), (yyvsp[(2) - (2)].string)); }
     break;
 
   case 6:
 /* Line 1787 of yacc.c  */
-#line 74 "ffilter/ffilter_gram.y"
-    { (yyval.node) = ff_new_node(scanner, filter, (yyvsp[(1) - (3)].node), FF_OP_OR, (yyvsp[(3) - (3)].node)); if ((yyval.node) == NULL) { YYABORT; }; }
+#line 77 "ffilter/ffilter_gram.y"
+    { snprintf((yyval.string), FF_MAX_STRING - 1, "%c%s", '|', (yyvsp[(2) - (2)].string)); }
     break;
 
   case 7:
 /* Line 1787 of yacc.c  */
-#line 75 "ffilter/ffilter_gram.y"
-    { (yyval.node) = (yyvsp[(2) - (3)].node); }
+#line 78 "ffilter/ffilter_gram.y"
+    { snprintf((yyval.string), FF_MAX_STRING - 1, "%c%s", '&', (yyvsp[(2) - (2)].string)); }
     break;
 
   case 8:
 /* Line 1787 of yacc.c  */
-#line 76 "ffilter/ffilter_gram.y"
-    { (yyval.node) = ff_new_leaf(scanner, filter, (yyvsp[(1) - (2)].string), FF_OP_EQ, (yyvsp[(2) - (2)].string)); if ((yyval.node) == NULL) { YYABORT; } }
+#line 79 "ffilter/ffilter_gram.y"
+    { snprintf((yyval.string), FF_MAX_STRING - 1, "%s %s", (yyvsp[(1) - (2)].string), (yyvsp[(2) - (2)].string)); }
     break;
 
   case 9:
 /* Line 1787 of yacc.c  */
-#line 77 "ffilter/ffilter_gram.y"
-    { (yyval.node) = ff_new_leaf(scanner, filter, (yyvsp[(1) - (3)].string), FF_OP_EQ, (yyvsp[(3) - (3)].string)); if ((yyval.node) == NULL) { YYABORT; } }
+#line 83 "ffilter/ffilter_gram.y"
+    { strncpy((yyval.string), (yyvsp[(1) - (1)].string), FF_MAX_STRING - 1); }
     break;
 
   case 10:
 /* Line 1787 of yacc.c  */
-#line 78 "ffilter/ffilter_gram.y"
-    { (yyval.node) = ff_new_leaf(scanner, filter, (yyvsp[(1) - (3)].string), FF_OP_LT, (yyvsp[(3) - (3)].string)); if ((yyval.node) == NULL) { YYABORT; } }
+#line 84 "ffilter/ffilter_gram.y"
+    { snprintf((yyval.string), FF_MAX_STRING - 1, "%s %s", (yyvsp[(1) - (2)].string), (yyvsp[(2) - (2)].string)); }
     break;
 
   case 11:
 /* Line 1787 of yacc.c  */
-#line 79 "ffilter/ffilter_gram.y"
-    { (yyval.node) = ff_new_leaf(scanner, filter, (yyvsp[(1) - (3)].string), FF_OP_GT, (yyvsp[(3) - (3)].string)); if ((yyval.node) == NULL) { YYABORT; } }
+#line 88 "ffilter/ffilter_gram.y"
+    { (yyval.node) = ff_new_node(scanner, filter, NULL, FF_OP_YES, NULL); if ((yyval.node) == NULL) { YYABORT; }; }
+    break;
+
+  case 12:
+/* Line 1787 of yacc.c  */
+#line 89 "ffilter/ffilter_gram.y"
+    { (yyval.node) = ff_new_node(scanner, filter, NULL, FF_OP_NOT, (yyvsp[(2) - (2)].node)); if ((yyval.node) == NULL) { YYABORT; }; }
+    break;
+
+  case 13:
+/* Line 1787 of yacc.c  */
+#line 90 "ffilter/ffilter_gram.y"
+    { (yyval.node) = ff_new_node(scanner, filter, (yyvsp[(1) - (3)].node), FF_OP_AND, (yyvsp[(3) - (3)].node)); if ((yyval.node) == NULL) { YYABORT; }; }
+    break;
+
+  case 14:
+/* Line 1787 of yacc.c  */
+#line 91 "ffilter/ffilter_gram.y"
+    { (yyval.node) = ff_new_node(scanner, filter, (yyvsp[(1) - (3)].node), FF_OP_OR, (yyvsp[(3) - (3)].node)); if ((yyval.node) == NULL) { YYABORT; }; }
+    break;
+
+  case 15:
+/* Line 1787 of yacc.c  */
+#line 92 "ffilter/ffilter_gram.y"
+    { (yyval.node) = (yyvsp[(2) - (3)].node); }
+    break;
+
+  case 16:
+/* Line 1787 of yacc.c  */
+#line 93 "ffilter/ffilter_gram.y"
+    { struct { char* name; char* field; char* cnst; } const_info[] =
+                        	{
+                            	{"inet", "inet", "4"},
+                        		{"inet6", "inet", "6"},
+                        		{"ipv4", "inet", "4"},
+                        		{"ipv6", "inet", "6"},
+                        		{NULL,NULL,NULL}
+                        	};
+                        	int x;
+                        	for(x = 0; const_info[x].name; x++){
+                        		if (!strcmp((yyvsp[(1) - (1)].string), const_info[x].name)){
+                        			(yyval.node) = ff_new_leaf(scanner, filter, const_info[x].field, FF_OP_EQ, const_info[x].cnst); if ((yyval.node) == NULL) { YYABORT; }
+                        			break;
+                        		}
+                        	}
+                        	if (!const_info[x].name) { YYABORT; };
+                        }
+    break;
+
+  case 17:
+/* Line 1787 of yacc.c  */
+#line 110 "ffilter/ffilter_gram.y"
+    { (yyval.node) = ff_new_leaf(scanner, filter, (yyvsp[(2) - (2)].string), FF_OP_EXIST, ""); if ((yyval.node) == NULL) { YYABORT; } }
+    break;
+
+  case 18:
+/* Line 1787 of yacc.c  */
+#line 111 "ffilter/ffilter_gram.y"
+    { (yyval.node) = ff_new_leaf(scanner, filter, (yyvsp[(1) - (3)].string), (yyvsp[(2) - (3)].t_uint), (yyvsp[(3) - (3)].string)); if ((yyval.node) == NULL) { YYABORT; } }
+    break;
+
+  case 19:
+/* Line 1787 of yacc.c  */
+#line 112 "ffilter/ffilter_gram.y"
+    { (yyval.node) = ff_new_leaf(scanner, filter, (yyvsp[(1) - (3)].string), FF_OP_IN, (yyvsp[(3) - (3)].node)); if ((yyval.node) == NULL) { YYABORT; } }
+    break;
+
+  case 20:
+/* Line 1787 of yacc.c  */
+#line 116 "ffilter/ffilter_gram.y"
+    { (yyval.node) = ff_new_mval(scanner, filter, (yyvsp[(1) - (2)].string), FF_OP_EQ, (yyvsp[(2) - (2)].node)); if ((yyval.node) == NULL) { YYABORT; } }
+    break;
+
+  case 21:
+/* Line 1787 of yacc.c  */
+#line 117 "ffilter/ffilter_gram.y"
+    { (yyval.node) = ff_new_mval(scanner, filter, (yyvsp[(1) - (2)].string), FF_OP_EQ, NULL); if ((yyval.node) == NULL) { YYABORT; } }
+    break;
+
+  case 22:
+/* Line 1787 of yacc.c  */
+#line 121 "ffilter/ffilter_gram.y"
+    { (yyval.t_uint) = FF_OP_ISSET; }
+    break;
+
+  case 23:
+/* Line 1787 of yacc.c  */
+#line 122 "ffilter/ffilter_gram.y"
+    { (yyval.t_uint) = FF_OP_EQ; }
+    break;
+
+  case 24:
+/* Line 1787 of yacc.c  */
+#line 123 "ffilter/ffilter_gram.y"
+    { (yyval.t_uint) = FF_OP_LT; }
+    break;
+
+  case 25:
+/* Line 1787 of yacc.c  */
+#line 124 "ffilter/ffilter_gram.y"
+    { (yyval.t_uint) = FF_OP_GT; }
+    break;
+
+  case 26:
+/* Line 1787 of yacc.c  */
+#line 125 "ffilter/ffilter_gram.y"
+    { (yyval.t_uint) = FF_OP_NOOP; }
     break;
 
 
 /* Line 1787 of yacc.c  */
-#line 1472 "ffilter/ffilter_gram.c"
+#line 1616 "ffilter/ffilter_gram.c"
       default: break;
     }
   /* User semantic actions sometimes alter yychar, and that requires
@@ -1700,6 +1844,6 @@ yyreturn:
 
 
 /* Line 2050 of yacc.c  */
-#line 82 "ffilter/ffilter_gram.y"
+#line 128 "ffilter/ffilter_gram.y"
 
 
