@@ -49,24 +49,46 @@ extern int ff2_debug;
      AND = 258,
      OR = 259,
      NOT = 260,
-     EQ = 261,
-     LT = 262,
-     GT = 263,
-     LP = 264,
-     RP = 265,
-     STRING = 266
+     ANY = 261,
+     EXIST = 262,
+     EQ = 263,
+     LT = 264,
+     GT = 265,
+     ISSET = 266,
+     LP = 267,
+     RP = 268,
+     LPS = 269,
+     RPS = 270,
+     IN = 271,
+     STRING = 272,
+     DIR = 273,
+     BIDIR_AND = 274,
+     BIDIR_OR = 275,
+     DIR_DIR_MAC = 276,
+     BAD_TOKEN = 277
    };
 #endif
 /* Tokens.  */
 #define AND 258
 #define OR 259
 #define NOT 260
-#define EQ 261
-#define LT 262
-#define GT 263
-#define LP 264
-#define RP 265
-#define STRING 266
+#define ANY 261
+#define EXIST 262
+#define EQ 263
+#define LT 264
+#define GT 265
+#define ISSET 266
+#define LP 267
+#define RP 268
+#define LPS 269
+#define RPS 270
+#define IN 271
+#define STRING 272
+#define DIR 273
+#define BIDIR_AND 274
+#define BIDIR_OR 275
+#define DIR_DIR_MAC 276
+#define BAD_TOKEN 277
 
 
 
@@ -74,7 +96,7 @@ extern int ff2_debug;
 typedef union YYSTYPE
 {
 /* Line 2053 of yacc.c  */
-#line 47 "ffilter/ffilter_gram.y"
+#line 45 "ffilter/ffilter_gram.y"
 
 	uint64_t	t_uint;
 	double		t_double;
@@ -83,7 +105,7 @@ typedef union YYSTYPE
 
 
 /* Line 2053 of yacc.c  */
-#line 87 "ffilter/ffilter_gram.h"
+#line 109 "ffilter/ffilter_gram.h"
 } YYSTYPE;
 # define YYSTYPE_IS_TRIVIAL 1
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
